package client.gui.character;

import client.gui.GuiConfirm;
import client.gui.GuiMenu;
import client.gui.element.ActButton;
import client.gui.element.ButtonCallback;
import client.gui.element.GuiList;
import client.gui.element.ListEntry;
import client.gui.element.NavButton;
import client.gui.element.PressType;
import client.gui.element.TransparentArea;
import client.renderer.Drawing;
import common.color.TextColor;
import common.entity.npc.PlayerCharacter;
import common.packet.CPacketAction;

public class GuiCharacters extends GuiList<GuiCharacters.CharacterEntry> implements ButtonCallback
{
	protected class CharacterEntry implements ListEntry
	{
	    private final PlayerCharacter character;
	    private final boolean initial;

	    protected CharacterEntry(PlayerCharacter character, boolean initial)
	    {
	    	this.character = character;
	    	this.initial = initial;
	    }

	    public void draw(int x, int y, int mouseX, int mouseY, boolean hovered)
	    {
	    	if(this.initial)
	    		Drawing.drawRect(x, y, 1, 36, 0xffaf0000);
	    	String str = this.character == null ? TextColor.BLUE + "[" + TextColor.CYAN + "+" + TextColor.BLUE + "]" :
	    		String.format(TextColor.GREEN + "Level " + TextColor.DGREEN + "%d " + TextColor.YELLOW + "%s " + TextColor.VIOLET + "%s" + TextColor.GRAY + " [%s%s" + TextColor.GRAY + "]", 
	    		character.level, character.type, character.name, character.align.color, character.align.display);
	    	String pos = this.character == null ? TextColor.BROWN + "Neuen Charakter erstellen" :
	    		String.format(TextColor.NEON + "%s " + TextColor.GRAY + "bei " + TextColor.ACID + "%d" + TextColor.GRAY + ", " + TextColor.ACID + "%d" + TextColor.GRAY + ", " + TextColor.ACID + "%d", 
	    		character.dim, character.pos.getX(), character.pos.getY(), character.pos.getZ());
            Drawing.drawText(str, x + 3, y, 0xffffffff);
            Drawing.drawText(pos, x + 3, y + 16, 0xffffffff);
	    }
	    
	    public void select(boolean dclick, int mx, int my)
	    {
	    	if(dclick)
	    		GuiCharacters.this.use(GuiCharacters.this.actionButtom, PressType.PRIMARY);
            GuiCharacters.this.updateButtons();
	    }
	}
	
	public static final GuiCharacters INSTANCE = new GuiCharacters();

    private TransparentArea descField;
    private ActButton actionButtom;
    private ActButton deleteButtom;
    
    private GuiCharacters() {
    }
    
    private void updateButtons() {
    	CharacterEntry entry = this.getSelected();
        this.descField.setText(entry == null ? "" : (entry.character == null ? "*neuer Charakter*" : (entry.character.info == null ? "*keine Beschreibung vorhanden*" : this.getSelected().character.info)));
    	this.actionButtom.setText(entry != null && entry.character == null ? "Charakter erstellen" : "Charakter spielen");
    	this.actionButtom.enabled = entry != null && !entry.initial;
    	this.deleteButtom.enabled = entry != null && entry.character != null && !entry.initial;
    }
    
    public void init(int width, int height)
    {
    	super.init(width, height);
        this.setDimensions(600, height, 32, height - 32);
        this.elements.clear();
        if(this.gm.getNetHandler() != null) {
        	int initialSelection = this.gm.getNetHandler().getSelectedCharacter();
        	for(PlayerCharacter character : this.gm.getNetHandler().getCharacterList()) {
        		this.elements.add(new CharacterEntry(initialSelection == this.elements.size() ? new PlayerCharacter(character.name, character.info, character.align, this.gm.player.worldObj.dimension.getFormattedName(false), this.gm.player.getPosition(), character.type, this.gm.player.experienceLevel) : character, initialSelection == this.elements.size()));
        	}
        	this.elements.add(new CharacterEntry(null, false));
        	this.setSelected(initialSelection);
        }
        this.descField = this.add(new TransparentArea(width - 390, 62, 380, height - 124, "", false));
        this.deleteButtom = this.add(new ActButton(width / 2 - 304, height - 28, 200, 24, this, "Charakter löschen"));
        this.actionButtom = this.add(new ActButton(width / 2 - 100, height - 28, 200, 24, this, ""));
        this.add(new NavButton(width / 2 + 104, height - 28, 200, 24, GuiMenu.INSTANCE, "Abbrechen"));
        this.updateButtons();
    }
    
	public String getTitle() {
		return "Charakter anpassen";
	}
	
    public int getListWidth()
    {
        return 560;
    }
    
    public int getSlotHeight()
    {
        return 36 + 4;
    }
    
    public void use(ActButton elem, PressType action) {
		CharacterEntry entry = GuiCharacters.this.getSelected();
		if(entry != null && GuiCharacters.this.gm.getNetHandler() != null) {
			if(elem == this.actionButtom) {
				if(entry.character == null)
					this.gm.getNetHandler().addToSendQueue(new CPacketAction(CPacketAction.Action.OPEN_EDITOR));
				else
					this.gm.getNetHandler().addToSendQueue(new CPacketAction(CPacketAction.Action.SELECT_CHARACTER, this.selectedElement));
			}
			else if(elem == this.deleteButtom && entry.character != null) {
				this.gm.displayGuiScreen(new GuiConfirm(new GuiConfirm.Callback() {
					public void confirm(boolean confirmed) {
						if(confirmed)
							GuiCharacters.this.gm.getNetHandler().addToSendQueue(new CPacketAction(CPacketAction.Action.DELETE_CHARACTER, GuiCharacters.this.selectedElement));
						GuiCharacters.this.gm.displayGuiScreen(GuiCharacters.this);
					}
				}, "Möchtest du diesen Charakter wirklich löschen?", "Der Fortschritt, die Gegenstände und die Historie von \"" + entry.character.name + "\" werden für imer verloren sein!", "Löschen", "Abbrechen"));
			}
		}
	}
}
