package common.packet;

import java.io.IOException;
import java.security.PrivateKey;
import java.security.PublicKey;

import javax.crypto.SecretKey;

import common.network.ILoginHandler;
import common.network.Packet;
import common.network.PacketBuffer;
import common.util.EncryptUtil;

public class LPacketStartEncrypt implements Packet<ILoginHandler> {
	private byte[] key = new byte[0];
	private byte[] token = new byte[0];

	public LPacketStartEncrypt() {
	}

	public LPacketStartEncrypt(SecretKey secret, PublicKey pubkey, byte[] token) {
		this.key = EncryptUtil.encryptData(pubkey, secret.getEncoded());
		this.token = EncryptUtil.encryptData(pubkey, token);
	}

	public void readPacketData(PacketBuffer buf) throws IOException {
		this.key = buf.readByteArray();
		this.token = buf.readByteArray();
	}

	public void writePacketData(PacketBuffer buf) throws IOException {
		buf.writeByteArray(this.key);
		buf.writeByteArray(this.token);
	}

	public void processPacket(ILoginHandler handler) {
		handler.processEncryption(this);
	}

	public SecretKey getKey(PrivateKey key) {
		return EncryptUtil.decryptSharedKey(key, this.key);
	}

	public byte[] getToken(PrivateKey key) {
		return key == null ? this.token : EncryptUtil.decryptData(key, this.token);
	}
}
