package common.util;

import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.EncodedKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;

import common.log.Log;

public class EncryptUtil {
	public static SecretKey createNewSharedKey() {
		try {
			KeyGenerator keygen = KeyGenerator.getInstance("AES");
			keygen.init(128);
			return keygen.generateKey();
		}
		catch(NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		}
	}

	public static KeyPair generateKeyPair() {
		try {
			KeyPairGenerator pairgen = KeyPairGenerator.getInstance("RSA");
			pairgen.initialize(2048);
			return pairgen.generateKeyPair();
		}
		catch(NoSuchAlgorithmException e) {
			Log.SYSTEM.error(e, "Konnte Schlüsselpaar nicht generiren");
			return null;
		}
	}
	
	public static PublicKey decodePublicKey(byte[] encoded) {
		try {
			EncodedKeySpec spec = new X509EncodedKeySpec(encoded);
			KeyFactory factory = KeyFactory.getInstance("RSA");
			return factory.generatePublic(spec);
		}
		catch(NoSuchAlgorithmException | InvalidKeySpecException e) {
			Log.SYSTEM.error(e, "Öffentlicher Schlüssel konnte nicht dekodiert werden");
			return null;
		}
	}

	public static SecretKey decryptSharedKey(PrivateKey key, byte[] secret) {
		return new SecretKeySpec(decryptData(key, secret), "AES");
	}

	public static byte[] encryptData(Key key, byte[] data) {
		return cipherOperation(Cipher.ENCRYPT_MODE, key, data);
	}

	public static byte[] decryptData(Key key, byte[] data) {
		return cipherOperation(Cipher.DECRYPT_MODE, key, data);
	}

	private static byte[] cipherOperation(int mode, Key key, byte[] data) {
		try {
			return createCipher(mode, key.getAlgorithm(), key).doFinal(data);
		}
		catch(IllegalBlockSizeException | BadPaddingException e) {
			Log.SYSTEM.error(e, "Konnte Daten nicht ver- oder entschlüsseln");
			return null;
		}
	}

	private static Cipher createCipher(int mode, String transformation, Key key) {
		try {
			Cipher cipher = Cipher.getInstance(transformation);
			cipher.init(mode, key);
			return cipher;
		}
		catch(InvalidKeyException | NoSuchAlgorithmException | NoSuchPaddingException e) {
			Log.SYSTEM.error(e, "Konnte Verschlüsselungsverfahren nicht initialisieren");
			return null;
		}
	}

	public static Cipher createCipher(int mode, Key key) {
		try {
			Cipher cipher = Cipher.getInstance("AES/CFB8/NoPadding");
			cipher.init(mode, (Key)key, (AlgorithmParameterSpec)(new IvParameterSpec(key.getEncoded())));
			return cipher;
		}
		catch(GeneralSecurityException e) {
			throw new RuntimeException(e);
		}
	}
}
